==> react-app-logic-15.4.0.tgz: package/src/gov/forms/useProposalDepositForm.ts <==
import { useCallback, useEffect, useMemo, useState } from 'react';
import BigNumber from 'bignumber.js';
import { useAsyncProcess, useAsyncStatuses } from '@onomy/react-utils';
import { DetailedFungibleAssetBalance, OnomyHubUtils } from '@onomy/hub';
import {
  useAssetBalance,
  useDetailedBalance,
  useGasForm,
  useOnomyHub,
  useFixedAmountInput,
  useProposal,
  useProposalMinDeposit,
  useStakingAssets,
} from '@onomy/react-hub';
import { Blockchain } from 'index';

type ProposalDepositResult = {
  proposalId: string;
  chain: Blockchain;
  balanceBefore: DetailedFungibleAssetBalance;
  balanceAfter: DetailedFungibleAssetBalance;
  amount: DetailedFungibleAssetBalance;
  gasAmount: DetailedFungibleAssetBalance;
};

export function useProposalDepositForm({
  autoSetNeededAmount: autoSetNeededAmountProp = false,
  ...props
}: {
  autoSetNeededAmount?: boolean;
} & Parameters<typeof useProposal>[0]) {
  const { hub } = useOnomyHub();

  const [attempt, txStatus, txReset] = useAsyncProcess();
  const [txDetails, setTxDetails] = useState<ProposalDepositResult | null>(null);

  const [proposal, proposalStatus] = useProposal(props);
  const [{ asset: govAsset, feeAsset }, assetStatus] = useStakingAssets(props.chain);

  const { asset = govAsset } = proposal ?? {};
  const [balance, balanceStatus] = useAssetBalance(asset);

  const [amountFixed, setAmountFixed] = useState('');
  const [autoSetNeededAmount, setAutoSetNeededAmount] = useState(autoSetNeededAmountProp);
  const toggleAutoSetNeededAmount = useCallback(() => setAutoSetNeededAmount(v => !v), []);

  const [amount, amountStatus] = useDetailedBalance(asset, amountFixed);
  const [minDepositAmount, minDepositAmountStatus] = useProposalMinDeposit(props.chain);

  // TODO: support multiple deposit denoms?
  const depositNeededFixed =
    minDepositAmount.amount - (proposal?.totalDeposits[0]?.amount ?? BigInt(0));
  const [depositNeeded, depositNeededStatus] = useDetailedBalance(
    asset,
    depositNeededFixed.toString(10)
  );

  useEffect(() => {
    if (autoSetNeededAmount) setAmountFixed(depositNeededFixed.toString(10));
  }, [depositNeededFixed, autoSetNeededAmount]);

  const [amountInput, setAmountInput] = useFixedAmountInput({
    value: amountFixed,
    onChangeValue: setAmountFixed,
  });

  const inputParams = useMemo(() => {
    return {
      proposalId: props.id,
      amountFixed: new BigNumber(amountFixed),
    };
  }, [props.id, amountFixed]);

  const gas = useGasForm(
    asset,
    feeAsset,
    async () => {
      if (!hub) return '0';
      const cosmosChain = OnomyHubUtils.getCosmosChain(hub.chains, props.chain);
      if (!cosmosChain) return '0';
      const res = await cosmosChain.client.feesForDepositToProposal(inputParams);
      const [denomAmount] = res.amount;
      return denomAmount.amount;
    },
    [hub, inputParams]
  );

  const loadingStatus = useAsyncStatuses([
    proposalStatus,
    assetStatus,
    balanceStatus,
    amountStatus,
    minDepositAmountStatus,
    depositNeededStatus,
  ]);

  const hasEnoughBalance = useMemo(() => {
    let total = amount.amount;

    if (amount.asset.id === gas.gasAmount.asset.id) total += gas.gasAmount.amount;
    return balance.amount >= total;
  }, [
    amount.amount,
    amount.asset.id,
    gas.gasAmount.asset.id,
    gas.gasAmount.amount,
    balance.amount,
  ]);

  const formErrors = useMemo(() => {
    const errors: {
      gas?: string;
      amountInput?: string;
      proposalId?: string;
    } = {};

    if (!inputParams.proposalId) {
      errors.proposalId = 'Proposal not set';
    }

    if (!inputParams.amountFixed) {
      errors.amountInput = 'Amount is required';
    } else {
      const amountFixedValue = new BigNumber(inputParams.amountFixed);
      if (amountFixedValue.lte(0) || amountFixedValue.isNaN()) {
        errors.amountInput = 'Amount must be greater than 0';
      }
    }

    if (!gas.hasEnoughGasBalance) errors.gas = `Insuffcient ${gas.gasAsset.meta.symbol}`;
    if (!hasEnoughBalance) errors.amountInput = `Insuffcient ${balance.asset.meta.symbol}`;

    return Object.keys(errors).length > 0 ? errors : null;
  }, [
    inputParams.proposalId,
    inputParams.amountFixed,
    gas.hasEnoughGasBalance,
    gas.gasAsset.meta.symbol,
    hasEnoughBalance,
    balance.asset.meta.symbol,
  ]);

  const canSubmit = useMemo(() => {
    return !formErrors && !loadingStatus.pending && !loadingStatus.error && !!gas.gasPrice;
  }, [formErrors, gas.gasPrice, loadingStatus]);

  const reset = useCallback(() => {
    setAmountFixed('');
    txReset();
    setTxDetails(null);
  }, [txReset]);

  const submit = useCallback(() => {
    return attempt(async () => {
      if (!hub || !canSubmit) return;
      const cosmosChain = OnomyHubUtils.getCosmosChain(hub.chains, props.chain);
      if (!cosmosChain) return;

      await cosmosChain.client.depositToProposal({
        ...inputParams,
        feeMultiplier: parseInt(gas.gasPrice, 10),
      });

      // Force update of proposal data
      await hub.proposals.forceGet(props.chain.name, inputParams.proposalId);
      const balanceAfter = await hub.balances.forceGet(balance.asset);

      const result = {
        proposalId: inputParams.proposalId,
        chain: props.chain,
        amount,
        gasAmount: gas.gasAmount,
        balanceBefore: balance,
        balanceAfter,
      };

      setTxDetails(result);
    });
  }, [
    attempt,
    hub,
    canSubmit,
    amount,
    balance,
    gas.gasAmount,
    gas.gasPrice,
    inputParams,
    props.chain,
  ]);

  return {
    loadingStatus,
    txStatus,
    txDetails,

    proposal,
    asset,
    balance,
    amount,
    minDepositAmount,
    depositNeeded,

    amountInput,
    autoSetNeededAmount,
    gas,

    setAmountInput,
    setAutoSetNeededAmount,
    toggleAutoSetNeededAmount,

    hasEnoughBalance,
    formErrors,
    canSubmit,

    reset,
    submit,
  };
}

==> react-app-logic-18.0.3.tgz: package/src/gov/forms/useProposalDepositForm.ts <==
import { useCallback, useEffect, useMemo, useState } from 'react';
import BigNumber from 'bignumber.js';
import { useAsyncProcess, useAsyncStatuses } from '@onomy/react-utils';
import { DetailedFungibleAssetBalance, OnomyHubUtils } from '@onomy/hub';
import {
  useDetailedBalance,
  useGasForm,
  useOnomyHub,
  useFixedAmountInput,
  useProposal,
  useProposalMinDeposit,
  useStakingAssets,
  useAssetBalanceUnlocked,
} from '@onomy/react-hub';
import { Blockchain } from 'index';

type ProposalDepositResult = {
  proposalId: string;
  chain: Blockchain;
  balanceBefore: DetailedFungibleAssetBalance;
  balanceAfter: DetailedFungibleAssetBalance;
  amount: DetailedFungibleAssetBalance;
  gasAmount: DetailedFungibleAssetBalance;
};

export function useProposalDepositForm({
  autoSetNeededAmount: autoSetNeededAmountProp = false,
  ...props
}: {
  autoSetNeededAmount?: boolean;
} & Parameters<typeof useProposal>[0]) {
  const { hub } = useOnomyHub();

  const [attempt, txStatus, txReset] = useAsyncProcess();
  const [txDetails, setTxDetails] = useState<ProposalDepositResult | null>(null);

  const [proposal, proposalStatus] = useProposal(props);
  const [{ asset: govAsset, feeAsset }, assetStatus] = useStakingAssets(props.chain);

  const { asset = govAsset } = proposal ?? {};
  const [balance, balanceStatus] = useAssetBalanceUnlocked(asset);

  const [amountFixed, setAmountFixed] = useState('');
  const [autoSetNeededAmount, setAutoSetNeededAmount] = useState(autoSetNeededAmountProp);
  const toggleAutoSetNeededAmount = useCallback(() => setAutoSetNeededAmount(v => !v), []);

  const [amount, amountStatus] = useDetailedBalance(asset, amountFixed);
  const [minDepositAmount, minDepositAmountStatus] = useProposalMinDeposit(props.chain);

  // TODO: support multiple deposit denoms?
  const depositNeededFixed =
    minDepositAmount.amount - (proposal?.totalDeposits[0]?.amount ?? BigInt(0));
  const [depositNeeded, depositNeededStatus] = useDetailedBalance(
    asset,
    depositNeededFixed.toString(10)
  );

  useEffect(() => {
    if (autoSetNeededAmount) setAmountFixed(depositNeededFixed.toString(10));
  }, [depositNeededFixed, autoSetNeededAmount]);

  const [amountInput, setAmountInput] = useFixedAmountInput({
    value: amountFixed,
    onChangeValue: setAmountFixed,
  });

  const inputParams = useMemo(() => {
    return {
      proposalId: props.id,
      amountFixed: new BigNumber(amountFixed),
    };
  }, [props.id, amountFixed]);

  const gas = useGasForm(
    asset,
    feeAsset,
    async () => {
      if (!hub) return '0';
      const cosmosChain = OnomyHubUtils.getCosmosChain(hub.chains, props.chain);
      if (!cosmosChain) return '0';
      const res = await cosmosChain.client.feesForDepositToProposal(inputParams);
      const [denomAmount] = res.amount;
      return denomAmount.amount;
    },
    [hub, inputParams]
  );

  const loadingStatus = useAsyncStatuses([
    proposalStatus,
    assetStatus,
    balanceStatus,
    amountStatus,
    minDepositAmountStatus,
    depositNeededStatus,
  ]);

  const hasEnoughBalance = useMemo(() => {
    let total = amount.amount;

    if (amount.asset.id === gas.gasAmount.asset.id) total += gas.gasAmount.amount;
    return balance.amount >= total;
  }, [
    amount.amount,
    amount.asset.id,
    gas.gasAmount.asset.id,
    gas.gasAmount.amount,
    balance.amount,
  ]);

  const formErrors = useMemo(() => {
    const errors: {
      gas?: string;
      amountInput?: string;
      proposalId?: string;
    } = {};

    if (!inputParams.proposalId) {
      errors.proposalId = 'Proposal not set';
    }

    if (!inputParams.amountFixed) {
      errors.amountInput = 'Amount is required';
    } else {
      const amountFixedValue = new BigNumber(inputParams.amountFixed);
      if (amountFixedValue.lte(0) || amountFixedValue.isNaN()) {
        errors.amountInput = 'Amount must be greater than 0';
      }
    }

    if (!gas.hasEnoughGasBalance) errors.gas = `Insuffcient ${gas.gasAsset.meta.symbol}`;
    if (!hasEnoughBalance) errors.amountInput = `Insuffcient ${balance.asset.meta.symbol}`;

    return Object.keys(errors).length > 0 ? errors : null;
  }, [
    inputParams.proposalId,
    inputParams.amountFixed,
    gas.hasEnoughGasBalance,
    gas.gasAsset.meta.symbol,
    hasEnoughBalance,
    balance.asset.meta.symbol,
  ]);

  const canSubmit = useMemo(() => {
    return !formErrors && !loadingStatus.pending && !loadingStatus.error && !!gas.gasPrice;
  }, [formErrors, gas.gasPrice, loadingStatus]);

  const reset = useCallback(() => {
    setAmountFixed('');
    txReset();
    setTxDetails(null);
  }, [txReset]);

  const submit = useCallback(() => {
    return attempt(async () => {
      if (!hub || !canSubmit) return;
      const cosmosChain = OnomyHubUtils.getCosmosChain(hub.chains, props.chain);
      if (!cosmosChain) return;

      await cosmosChain.client.depositToProposal({
        ...inputParams,
        feeMultiplier: parseInt(gas.gasPrice, 10),
      });

      // Force update of proposal data
      await hub.proposals.forceGet(props.chain.name, inputParams.proposalId);
      const balanceAfter = await hub.balances.forceGet(balance.asset);

      const result = {
        proposalId: inputParams.proposalId,
        chain: props.chain,
        amount,
        gasAmount: gas.gasAmount,
        balanceBefore: balance,
        balanceAfter,
      };

      setTxDetails(result);
    });
  }, [
    attempt,
    hub,
    canSubmit,
    amount,
    balance,
    gas.gasAmount,
    gas.gasPrice,
    inputParams,
    props.chain,
  ]);

  return {
    loadingStatus,
    txStatus,
    txDetails,

    proposal,
    asset,
    balance,
    amount,
    minDepositAmount,
    depositNeeded,

    amountInput,
    autoSetNeededAmount,
    gas,

    setAmountInput,
    setAutoSetNeededAmount,
    toggleAutoSetNeededAmount,

    hasEnoughBalance,
    formErrors,
    canSubmit,

    reset,
    submit,
  };
}
